@@ -25,19 +25,16 @@ High-level Pipelines with plugable back-ends
 
 import gobject
 import gst
+from elements.smartscale import SmartVideoScale
 
 class SmartBin(gst.Pipeline):
     """
     High-level pipeline with playing/encoding ready places
     It also has length information
     """
-    length = 0
-    has_video = False
-    has_audio = False
-    width = 0
-    height = 0
 
-    def __init__(self, name, displayname=""):
+    def __init__(self, name, displayname="", has_video=False, has_audio=False,
+                 length=0, width=0, height=0):
         """
         @type name: string
         @param name: The name of the SmartBin (for internal use)
@@ -46,8 +43,17 @@ class SmartBin(gst.Pipeline):
         """
         gst.log('name : %s, displayname : %s' % (name, displayname))
         gobject.GObject.__init__(self)
+        self.length = length
+        self.has_video = has_video
+        self.has_audio = has_audio
+        self.width = width
+        self.height = height
         self.name = name
         self.displayname = displayname
+
+        self.vtee = None
+        self.atee = None
+
         self.set_name(name)
         # Until  basetransform issues are fixed, we use an identity instead
         # of a tee
@@ -191,13 +197,13 @@ class SmartBin(gst.Pipeline):
 
         # set sync=false on the videosink
         #self.getRealVideoSink().set_property("sync", False)
-        
+
         self.debug("linking vtee to ecnthread:vsink")
         try:
             self.vtee.get_pad("src").link(self.encthread.get_pad("vsink"))
         except:
             return False
-        
+
         self.debug("linking atee to encthread:asink")
         try:
             self.atee.get_pad("src").link(self.encthread.get_pad("asink"))
@@ -214,7 +220,7 @@ class SmartBin(gst.Pipeline):
     def stopRecording(self):
         """ stop the recording, removing the encoding thread """
         self.set_state(gst.STATE_PAUSED)
-        
+
         if self.encthread:
             apad = self.encthread.get_pad("vsink")
             apad.get_peer().unlink(apad)
@@ -231,56 +237,126 @@ class SmartBin(gst.Pipeline):
 
         self.getRealVideoSink().set_property("sync", True)
 
+    def getSettings(self):
+        """ Return the ExportSettings for the bin """
+        return None
+
+    def _debugProbe(self, unused_pad, data, categoryname):
+        if isinstance(data, gst.Buffer):
+            self.log("%s\tBUFFER timestamp:%s duration:%s size:%d" % (categoryname,
+                                                                      gst.TIME_ARGS(data.timestamp),
+                                                                      gst.TIME_ARGS(data.duration),
+                                                                      data.size))
+            if not data.flag_is_set(gst.BUFFER_FLAG_DELTA_UNIT):
+                self.log("%s\tKEYFRAME" % categoryname)
+        else:
+            self.log("%s\tEVENT %s" % (categoryname, data))
+        return True
+
     def _makeEncThread(self, uri, settings=None):
+        """ Construct the encoding bin according to the given setting. """
         # TODO : verify if encoders take video/x-raw-yuv and audio/x-raw-int
+        # TODO : use video/audio settings !
+        # TODO : Check if we really do both audio and video !
+
         if not settings:
-            if isinstance(self, SmartTimelineBin):
-                settings = self.project.settings
-            else:
+            setting = self.getSettings()
+            if not settings:
+                self.error("No settings available to create the Encoding Thread")
                 return None
-        ainq = gst.element_factory_make("queue", "ainq")
-        aoutq = gst.element_factory_make("queue", "aoutq")
-        vinq = gst.element_factory_make("queue", "vinq")
-        voutq = gst.element_factory_make("queue", "voutq")
-        aenc = gst.element_factory_make(settings.aencoder ,"aenc")
-        for prop, value in settings.acodecsettings.iteritems():
-            aenc.set_property(prop, value)
-        venc = gst.element_factory_make(settings.vencoder, "venc")
-        for prop, value in settings.vcodecsettings.iteritems():
-            venc.set_property(prop, value)
+
+        thread = gst.Bin("encthread")
+
+        ##
+        ## Muxer/FileSink part
+        ##
+
         mux = gst.element_factory_make(settings.muxer, "mux")
+        # set properties on the muxer
         for prop, value in settings.containersettings.iteritems():
             mux.set_property(prop, value)
         fsink = gst.element_make_from_uri(gst.URI_SINK, uri, "fsink")
+        thread.add(mux, fsink)
+        mux.link(fsink)
 
+        ##
+        ## Audio part
+        ##
 
-        thread = gst.Bin("encthread")
-        thread.add(mux, fsink, aoutq, voutq)
-
-        thread.add(ainq, aenc)
-       
+        ainq = gst.element_factory_make("queue", "ainq")
+        aident = gst.element_factory_make("identity", "aident")
+        aident.props.single_segment = True
         aconv = gst.element_factory_make("audioconvert", "aconv")
-        thread.add(aconv)
-        ainq.link(aconv)
-        aconv.link(aenc)
-        aenc.link(aoutq)
+        ares = gst.element_factory_make("audioresample", "ares")
+        arate = gst.element_factory_make("audiorate", "arate")
+        aenc = gst.element_factory_make(settings.aencoder ,"aenc")
+        # set properties on the encoder
+        for prop, value in settings.acodecsettings.iteritems():
+            aenc.set_property(prop, value)
+        aoutq = gst.element_factory_make("queue", "aoutq")
 
-        thread.add(vinq, venc)
+        # add and link all required audio elements
+        thread.add(ainq, aident, aconv, ares, arate, aenc, aoutq)
+        gst.element_link_many(ainq, aident, aconv, ares, arate)
+
+        # link to encoder using the settings caps
+        self.log("About to link encoder with settings pads")
+        try:
+            arate.link(aenc, settings.getAudioCaps())
+        except:
+            self.error("The audio encoder doesn't accept the audio settings")
+            return None
+        gst.element_link_many(aenc, aoutq, mux)
+
+        # ghost sinkpad
+        aghost = gst.GhostPad("asink", ainq.get_pad("sink"))
+        aghost.set_active(True)
+        thread.add_pad(aghost)
+
+##         aenc.get_pad("sink").add_data_probe(self._debugProbe, "aenc-sink")
+##         aenc.get_pad("src").add_data_probe(self._debugProbe, "aenc-src")
+
+        ##
+        ## Video part
+        ##
+
+        vinq = gst.element_factory_make("queue", "vinq")
+        vident = gst.element_factory_make("identity", "vident")
+        vident.props.single_segment = True
         csp = gst.element_factory_make("ffmpegcolorspace", "csp")
-        thread.add(csp)
-        vinq.link(csp)
-        csp.link(venc)
-        venc.link(voutq)
+        vrate = gst.element_factory_make("videorate", "vrate")
+        vscale = SmartVideoScale()
+        venc = gst.element_factory_make(settings.vencoder, "venc")
+        # set properties on the encoder
+        for prop, value in settings.vcodecsettings.iteritems():
+            venc.set_property(prop, value)
+        voutq = gst.element_factory_make("queue", "voutq")
 
-        thread.add_pad(gst.GhostPad("vsink", vinq.get_pad("sink")))
-        thread.add_pad(gst.GhostPad("asink", ainq.get_pad("sink")))
+        # add and link all required video elements
+        thread.add(vinq, vident, csp, vrate, vscale, venc, voutq)
+        caps = settings.getVideoCaps()
+        vscale.set_caps(caps)
+        gst.element_link_many(vinq, vident, csp, vscale, vrate)
+
+        # link to encoder using the settings caps
+        self.log("About to link encoder with settings pads")
+        try:
+            vrate.link(venc, caps)
+        except:
+            self.error("The video encoder doesn't accept the video settings")
+            return None
+        gst.element_link_many(venc, voutq, mux)
+
+        # ghost sinkpad
+        vghost = gst.GhostPad("vsink", vinq.get_pad("sink"))
+        vghost.set_active(True)
+        thread.add_pad(vghost)
+
+##         vrate.get_pad("sink").add_data_probe(self._debugProbe, "before-vrate")
+##         vrate.get_pad("src").add_data_probe(self._debugProbe, "after-vrate")
 
         thread.filesink = fsink
 
-        aoutq.link(mux)
-        voutq.link(mux)
-        mux.link(fsink)
-
         return thread
 
 class SmartFileBin(SmartBin):
@@ -291,16 +367,20 @@ class SmartFileBin(SmartBin):
     def __init__(self, factory):
         gst.log("new SmartFileBin for factory:%s, audio:%s, video:%s" % (factory, factory.is_audio, factory.is_video))
         self.factory = factory
-        self.has_video = factory.is_video
-        self.has_audio = factory.is_audio
-        self.length = factory.length
         if self.factory.video_info:
             struct = self.factory.video_info[0]
-            self.height = struct["height"]
-            self.width = struct["width"]
+            height = struct["height"]
+            width = struct["width"]
+        else:
+            height = 0
+            width = 0
         self.source = self.factory.makeBin()
         SmartBin.__init__(self, "smartfilebin-" + factory.name,
-                          displayname=factory.displayname)
+                          displayname=factory.displayname,
+                          has_video = factory.is_video,
+                          has_audio = factory.is_audio,
+                          width = width, height = height,
+                          length = factory.length)
 
     def _addSource(self):
         self.add(self.source)
@@ -313,16 +393,25 @@ class SmartFileBin(SmartBin):
         # connect to good tee
         self.debug("SmartFileBin's source has a new pad: %s %s" % (pad , pad.get_caps().to_string()))
         if pad.get_caps().to_string().startswith("audio"):
-            pad.link(self.atee.get_pad("sink"))
+            if not self.atee:
+                self.warning("Got new audio pad, but we didn't discover one previously !")
+            else:
+                pad.link(self.atee.get_pad("sink"))
         elif pad.get_caps().to_string().startswith("video"):
-            pad.link(self.vtee.get_pad("sink"))
+            if not self.vtee:
+                self.warning("Got new video pad, but we didn't discover one previously !")
+            else:
+                pad.link(self.vtee.get_pad("sink"))
 
     def _binRemovedDecodedPadCb(self, unused_bin, pad):
-        if pad.get_caps().to_string().startswith("audio"):
+        if pad.get_caps().to_string().startswith("audio") and self.atee:
             pad.unlink(self.atee.get_pad("sink"))
-        elif pad.get_caps().to_string().startswith("video"):
+        elif pad.get_caps().to_string().startswith("video") and self.vtee:
             pad.unlink(self.vtee.get_pad("sink"))
 
+    def getSettings(self):
+        return self.factory.getExportSettings()
+
     def do_destroy(self):
         self.info("destroyed")
         self.factory.binIsDestroyed(self.source)
@@ -336,20 +425,20 @@ class SmartTimelineBin(SmartBin):
     def __init__(self, project):
         gst.log("new SmartTimelineBin for project %s" % project)
         self.project = project
-        
-        # TODO : change this to use the project settings
-        self.has_video = True
-        self.has_audio = True
 
-        self.width = project.settings.videowidth
-        self.height = project.settings.videoheight
+        settings = project.getSettings()
         self.log("source is %s" % project.timeline.timeline)
         self.source = project.timeline.timeline
-        self.project.settings.connect("settings-changed", self._settingsChangedCb)
+        self.project.connect("settings-changed", self._settingsChangedCb)
         project.timeline.videocomp.connect("start-duration-changed", self._startDurationChangedCb)
-        self.length = project.timeline.videocomp.duration
+
+        # TODO : change has_audio/has_video to project settings value
         SmartBin.__init__(self, "project-" + project.name,
-                          displayname = "Project: " + project.name)
+                          displayname = "Project: " + project.name,
+                          has_video=True, has_audio=True,
+                          width=settings.videowidth,
+                          height=settings.videoheight,
+                          length=project.timeline.videocomp.duration)
 
     def _addSource(self):
         self.add(self.source)
@@ -358,7 +447,8 @@ class SmartTimelineBin(SmartBin):
         self.source.connect("pad-added", self._newPadCb)
         self.source.connect("pad-removed", self._removedPadCb)
 
-    def _settingsChangedCb(self, settings):
+    def _settingsChangedCb(self, project):
+        settings = project.getSettings()
         self.width = settings.videowidth
         self.height = settings.videoheight
 
@@ -380,6 +470,8 @@ class SmartTimelineBin(SmartBin):
         self.info("smart timeline bin: start duration changed %d %d" %( start, duration ))
         self.length = duration
 
+    def getSettings(self):
+        return self.project.getSettings()
 
 class SmartDefaultBin(SmartBin):
     """
@@ -393,11 +485,8 @@ class SmartDefaultBin(SmartBin):
         self.silence = gst.element_factory_make("audiotestsrc", "silence")
         self.videotestsrc.set_property("pattern", 2)
         self.silence.set_property("wave", 4)
-        self.has_audio = True
-        self.has_video = True
-        self.width = 720
-        self.height = 576
-        SmartBin.__init__(self, "smartdefaultbin")
+        SmartBin.__init__(self, "smartdefaultbin", has_video=True, has_audio=True,
+                          width=720, height=576)
 
     def _addSource(self):
         self.add(self.videotestsrc, self.silence)
@@ -408,4 +497,3 @@ class SmartDefaultBin(SmartBin):
         self.videotestsrc.get_pad("src").link(self.vtee.get_pad("sink"))
         self.silence.get_pad("src").link(self.atee.get_pad("sink"))
         self.debug("finished connecting sources")
-
