@@ -26,9 +26,11 @@ Where all gstreamer pipelines play.
 
 import gobject
 import gst
-from bin import SmartBin, SmartDefaultBin, SmartFileBin
+from bin import SmartBin, SmartDefaultBin, SmartFileBin, SmartTimelineBin
 from utils import bin_contains
 
+from gettext import gettext as _
+
 class PlayGround(gobject.GObject):
     """
     Holds all the applications pipelines.
@@ -46,6 +48,7 @@ class PlayGround(gobject.GObject):
       bin-removed : The given bin was removed from the playground
       error : An error was seen (two strings : reason, details)
       position : Updated position (SmartBin, position)
+      element-message : messages received of type gst.MESSAGE_ELEMENT
     """
 
     __gsignals__ = {
@@ -66,7 +69,10 @@ class PlayGround(gobject.GObject):
                     ( gobject.TYPE_STRING, gobject.TYPE_STRING )),
         "position" : ( gobject.SIGNAL_RUN_LAST,
                        gobject.TYPE_NONE,
-                       ( gobject.TYPE_PYOBJECT, gobject.TYPE_UINT64 ))
+                       ( gobject.TYPE_PYOBJECT, gobject.TYPE_UINT64 )),
+        "element-message" : ( gobject.SIGNAL_RUN_LAST,
+                              gobject.TYPE_NONE,
+                              ( gst.Message, ))
         }
 
     def __init__(self):
@@ -74,15 +80,16 @@ class PlayGround(gobject.GObject):
         gobject.GObject.__init__(self)
         # List of used pipelines
         self.pipelines = []
-        
+
         self.vsinkthread = None
         self.asinkthread = None
-        
+
         # Defaut pipeline if no other pipeline is playing
         self.default = SmartDefaultBin()
         bus = self.default.get_bus()
         bus.add_signal_watch()
         bus.connect("message", self._busMessageCb, self.default)
+        bus.set_sync_handler(self._busSyncMessageHandler)
 
         # Current playing pipeline
         self.current = None
@@ -92,7 +99,7 @@ class PlayGround(gobject.GObject):
         self.tempsmartbin = None
         self.cur_state_signal = None
         self.cur_eos_signal = None
-        
+
         self.state = gst.STATE_READY
 
         # FIXME : THIS IS CRACK, THIS IS CRACK
@@ -119,6 +126,7 @@ class PlayGround(gobject.GObject):
         bus = pipeline.get_bus()
         bus.add_signal_watch()
         bus.connect("message", self._busMessageCb, pipeline)
+        bus.set_sync_handler(self._busSyncMessageHandler)
         self.emit("bin-added", pipeline)
         return True
 
@@ -133,6 +141,7 @@ class PlayGround(gobject.GObject):
 
         bus = pipeline.get_bus()
         bus.remove_signal_watch()
+        bus.set_sync_handler(None)
 
         if pipeline.set_state(gst.STATE_READY) == gst.STATE_CHANGE_FAILURE:
             return False
@@ -145,7 +154,7 @@ class PlayGround(gobject.GObject):
 
     def switchToPipeline(self, pipeline):
         """
-        Switch to the given pipeline for play output.        
+        Switch to the given pipeline for play output.
         Returns True if the switch was possible.
         """
         if self.current == pipeline:
@@ -190,6 +199,21 @@ class PlayGround(gobject.GObject):
         gst.debug("switching to default")
         return self.switchToPipeline(self.default)
 
+    def switchToTimeline(self):
+        """
+        switch to the timeline.
+        Warning : if there's more than one timeline controlled, the first one
+        will be taken.
+        """
+        if isinstance(self.current, SmartTimelineBin):
+            # fast path
+            return True
+        for pipeline in self.pipelines:
+            if isinstance(pipeline, SmartTimelineBin):
+                self.switchToPipeline(pipeline)
+                return True
+        return False
+
     def setVideoSinkThread(self, vsinkthread):
         """ sets the video sink thread """
         gst.debug("video sink thread : %s" % vsinkthread)
@@ -239,6 +263,18 @@ class PlayGround(gobject.GObject):
         tempbin = SmartFileBin(factory)
         return self._playTemporaryBin(tempbin)
 
+    def getCurrentTimePosition(self):
+        """
+        Returns the current position of the current bin in gst.FORMAT_TIME
+        """
+        try:
+            cur, format = self.current.query_position(gst.FORMAT_TIME)
+        except:
+            self.current.warning("Couldn't get position in time")
+            cur = 0
+        self.current.log("Returning %s" % gst.TIME_ARGS(cur))
+        return cur
+
     def seekInCurrent(self, value, format=gst.FORMAT_TIME):
         """
         Seek to the given position in the current playing bin.
@@ -256,6 +292,9 @@ class PlayGround(gobject.GObject):
             gobject.source_remove(self._positiontimeoutid)
             self._positiontimeoutid = 0
 
+        if value < 0:
+            value = long(0)
+
         # actual seeking
         res = target.seek(1.0, format, gst.SEEK_FLAG_FLUSH,
                           gst.SEEK_TYPE_SET, value,
@@ -266,7 +305,7 @@ class PlayGround(gobject.GObject):
         gst.debug("Seeking to %s succeeded" % gst.TIME_ARGS (value))
 
         self.emit('position', self.current, value)
-        
+
         return True
 
     def shutdown(self):
@@ -286,11 +325,7 @@ class PlayGround(gobject.GObject):
 
     def _checkTimeCb(self):
         gst.log("Checking time, current:%r" % self.current)
-        try:
-            cur, format = self.current.query_position(gst.FORMAT_TIME)
-        except:
-            self.current.warning("Couldn't get position in time")
-            cur = 0
+        cur = self.getCurrentTimePosition()
         if cur == -1:
             gst.warning("seeking returned GST_CLOCK_TIME_NONE")
         else:
@@ -313,17 +348,20 @@ class PlayGround(gobject.GObject):
             if (not self.current == self.default) and message.src == self.current:
                 if pending == gst.STATE_VOID_PENDING:
                     if (not self._positiontimeoutid) and newstate in [gst.STATE_PLAYING]:
-			self._checkTimeCb()
+                        self._checkTimeCb()
                         self._positiontimeoutid = gobject.timeout_add(300, self._checkTimeCb)
                     self.emit("current-state", newstate)
         elif message.type == gst.MESSAGE_ERROR:
             error, detail = message.parse_error()
             self._handleError(error, detail, message.src)
-        elif message.type == gst.MESSAGE_WARNING:
-            error, detail = message.parse_warning()
-            self._handleError(error, detail, message.src)
 
 
+    def _busSyncMessageHandler(self, unused_bus, message):
+        if message.type == gst.MESSAGE_ELEMENT:
+            # handle element message synchronously
+            self.emit('element-message', message)
+        return gst.BUS_PASS
+
     #
     # Error handling
     #
@@ -341,31 +379,31 @@ class PlayGround(gobject.GObject):
                                                                       type(gerror.domain), gerror.message))
         if bin_contains(self.vsinkthread, source):
             if gerror.domain == 'gst-resource-error-quark' and gerror.code == gst.RESOURCE_ERROR_BUSY:
-                self.emit("error", "Video output is busy",
-                          "Please check that your video output device isn't already used by another application")
+                self.emit("error", _("Video output is busy"),
+                          _("Please check that your video output device isn't already used by another application"))
             else:
-                self.emit("error", "Video output problem",
-                          "There is a problem with your video output device")
+                self.emit("error", _("Video output problem"),
+                          _("There is a problem with your video output device"))
         elif bin_contains(self.asinkthread, source):
             if gerror.domain == 'gst-resource-error-quark' and gerror.code == gst.RESOURCE_ERROR_BUSY:
-                self.emit("error", "Audio output device is busy",
-                          "Please check that your audio output device isn't already used by another application.")
+                self.emit("error", _("Audio output device is busy"),
+                          _("Please check that your audio output device isn't already used by another application."))
             else:
-                self.emit("error", "Audio output problem",
-                          "There is a problem with your audio output device")
+                self.emit("error", _("Audio output problem"),
+                          _("There is a problem with your audio output device"))
         else:
             self.emit("error", gerror.message, detail)
-            
-        
 
-    def _handleWarning(self, error, detail, source):
+
+
+    def _handleWarning(self, gerror, detail, source):
         """
         Uses the information from the Gerror, detail and source to
         create meaningful warning messages for the User Interface.
         """
         gst.warning("gerror:%s , detail:%s , source:%s" % (gerror, detail, source))
         gst.warning("GError : code:%s, domain:%s, message:%s" % (gerror.code, gerror.domain, gerror.message))
-        
+
 
     #
     # playing proxy functions
@@ -384,7 +422,7 @@ class PlayGround(gobject.GObject):
         ret = self.current.set_state(self.state)
         gst.debug("change state returned %s" % ret)
         return ret
- 
+
     def pause(self):
         """
         Set the current Pipeline to Pause.
@@ -409,15 +447,33 @@ class PlayGround(gobject.GObject):
         """ play the current pipeline backwards """
         pass
 
+    ## Frame by frame seeking
+    # Goes forward/backward in the stream by one video frame
+    # For timelines, it will use the current export setting framerate as the
+    # framerate to use for calculatiosn. Not perfect, but still very close.
+
     def forward_one(self):
         """ forward the current pipeline by one video frame """
-        pass
+        if not self.current or not type(self.current) in [SmartTimelineBin, SmartFileBin]:
+            gst.warning("you can only do frame seeking on files or timelines")
+            return
+        pos = self.getCurrentTimePosition()
+        framerate = self.current.getSettings().videorate
+        target = pos + framerate.denom * gst.SECOND / framerate.num
+        gst.log("current position is %s , seeking to %s" % (
+            gst.TIME_ARGS(pos),
+            gst.TIME_ARGS(target)))
+        self.seekInCurrent(target)
 
     def backward_one(self):
         """ rewind the current pipeline by one video frame """
-        pass
-
-    def seek(self, time):
-        """ seek in the current pipeline """
-        pass
-
+        if not self.current or not type(self.current) in [SmartTimelineBin, SmartFileBin]:
+            gst.warning("you can only do frame seeking on files or timelines")
+            return
+        pos = self.getCurrentTimePosition()
+        framerate = self.current.getSettings().videorate
+        target = pos - framerate.denom * gst.SECOND / framerate.num
+        gst.log("current position is %s , seeking to %s" % (
+            gst.TIME_ARGS(pos),
+            gst.TIME_ARGS(target)))
+        self.seekInCurrent(target)
