@@ -40,6 +40,10 @@ class SourceList(gobject.GObject):
                 The given uri is not a media file
     _ tmp-is-ready (FileSourceFactory) :
                 The temporary uri given to the SourceList is ready to use.
+    _ ready :
+                No more files are being discovered/added
+    _ starting :
+                Some files are being discovered/added
     """
 
     __gsignals__ = {
@@ -51,10 +55,17 @@ class SourceList(gobject.GObject):
                           (gobject.TYPE_STRING, )),
         "not_media_file" : (gobject.SIGNAL_RUN_LAST,
                             gobject.TYPE_NONE,
-                            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
+                            (gobject.TYPE_STRING, gobject.TYPE_STRING,
+                             gobject.TYPE_STRING)),
         "tmp_is_ready": (gobject.SIGNAL_RUN_LAST,
                          gobject.TYPE_NONE,
-                         (gobject.TYPE_PYOBJECT, ))
+                         (gobject.TYPE_PYOBJECT, )),
+        "ready" : ( gobject.SIGNAL_RUN_LAST,
+                    gobject.TYPE_NONE,
+                    ( )),
+        "starting" : ( gobject.SIGNAL_RUN_LAST,
+                       gobject.TYPE_NONE,
+                       ( ))
         }
 
     def __init__(self, project):
@@ -66,6 +77,8 @@ class SourceList(gobject.GObject):
         self.discoverer = Discoverer(self.project)
         self.discoverer.connect("not_media_file", self._notMediaFileCb)
         self.discoverer.connect("finished_analyzing", self._finishedAnalyzingCb)
+        self.discoverer.connect("starting", self._discovererStartingCb)
+        self.discoverer.connect("ready", self._discovererReadyCb)
 
     def __contains__(self, uri):
         return self.sources.__contains__(uri)
@@ -90,7 +103,7 @@ class SourceList(gobject.GObject):
     def __iter__(self):
         """ returns an (uri, factory) iterator over the sources """
         return self.sources.iteritems()
-        
+
     def addUri(self, uri):
         """ Add the uri to the list of sources, will be discovered """
         # here we add the uri and emit a signal
@@ -141,11 +154,17 @@ class SourceList(gobject.GObject):
             self.sources[factory.name] = factory
             self.emit("file-added", factory)
 
-    def _notMediaFileCb(self, unused_discoverer, uri, reason):
+    def _notMediaFileCb(self, unused_discoverer, uri, reason, extra):
         # callback from the discoverer's 'not_media_file' signal
         # remove it from the list
-        self.emit("not_media_file", uri, reason)
+        self.emit("not_media_file", uri, reason, extra)
         if uri in self.sources and not self.sources[uri]:
             del self.sources[uri]
         elif uri in self.tempsources:
             del self.tempsources[uri]
+
+    def _discovererStartingCb(self, unused_discoverer):
+        self.emit("starting")
+
+    def _discovererReadyCb(self, unused_discoverer):
+        self.emit("ready")
