@@ -26,7 +26,6 @@ Timeline Composition object
 import gobject
 import gst
 
-from pitivi.settings import ExportSettings
 from source import TimelineSource
 from objects import BrotherObjects, MEDIA_TYPE_AUDIO
 
@@ -45,7 +44,7 @@ class EffectsLayer(Layer):
     """
 
     def __init__(self, priority):
-        Layer.__initi__(self)
+        Layer.__init__(self)
         self._priority = priority
         self._effects = []
 
@@ -62,7 +61,7 @@ class SourcesLayer(Layer):
     """
 
     def __init__(self, minprio, maxprio):
-        Layer.__initi__(self)
+        Layer.__init__(self)
         self._minprio = minprio
         self._maxprio = maxprio
         self._sources = []
@@ -94,13 +93,13 @@ class TimelineComposition(TimelineSource):
 
     _ Provides a "condensed list" of the objects contained within
       _ Allows to quickly show a top-level view of the composition
-    
+
     * Sandwich view example (top: high priority):
-	     [ Global Simple Effect(s) (RGB, YUV, Speed,...)	]
-	     [ Simple Effect(s), can be several layers		]
-	     [ Complex Effect(s), non-overlapping		]
-	     [ Transition(s), non-overlapping			]
-	     [ Layers of sources				]
+             [ Global Simple Effect(s) (RGB, YUV, Speed,...)    ]
+             [ Simple Effect(s), can be several layers          ]
+             [ Complex Effect(s), non-overlapping               ]
+             [ Transition(s), non-overlapping                   ]
+             [ Layers of sources                                ]
 
     * Properties:
       _ Global Simple Effect(s) (Optionnal)
@@ -190,7 +189,7 @@ class TimelineComposition(TimelineSource):
         # list of layers of simple effects (order: priority, then time)
         self.condensed = [] # list of sources/transitions seen from a top-level view
         # each layer contains (min priority, max priority, list objects)
-        #sources = [(2048, 2060, [])] 
+        #sources = [(2048, 2060, [])]
         self.sources = [(2048, 2060, [])]
         self.defaultSource = None
         TimelineSource.__init__(self, **kw)
@@ -210,7 +209,7 @@ class TimelineComposition(TimelineSource):
         return gst.element_factory_make("gnlcomposition", "composition-" + self.name)
 
     # global effects
-    
+
     def addGlobalEffect(self, global_effect, order, auto_linked=True):
         """
         add a global effect
@@ -231,7 +230,7 @@ class TimelineComposition(TimelineSource):
         raise NotImplementedError
 
     # simple effects
-    
+
     def addSimpleEffect(self, simple_effect, order, auto_linked=True):
         """
         add a simple effect
@@ -281,7 +280,7 @@ class TimelineComposition(TimelineSource):
                 return list2[:]
             if not len(list2):
                 return list1[:]
-            
+
             res = list1[:]
 
             # find the objects in list2 that go under list1 and insert them at
@@ -296,8 +295,8 @@ class TimelineComposition(TimelineSource):
                     res.append(obj)
             self.gnlobject.info("returning %s" % res)
             return res
-                
-            
+
+
         lists = [x[2] for x in self.sources]
         lists.insert(0, self.transitions)
         return reduce(condensed_sum, lists)
@@ -390,14 +389,14 @@ class TimelineComposition(TimelineSource):
                 i = i + 1
             object.gnlobject.set_property("priority", sources[0])
             slist.insert(i, object)
-            
+
         # TODO : add functionnality to add above/under
         # For the time being it's hardcoded to a single layer
         position = 1
 
         # add it to the correct self.sources[position]
         my_add_sorted(self.sources[position-1], source)
-        
+
         # add it to self.gnlobject
         self.gnlobject.info("adding %s to our composition" % source.gnlobject)
         self.gnlobject.add(source.gnlobject)
@@ -420,7 +419,7 @@ class TimelineComposition(TimelineSource):
                 to the linked composition with the same parameters
         """
         self.gnlobject.info("source %s , position:%d, self.sources:%s" %(source, position, self.sources))
-        
+
         self._addSource(source, position)
 
         # if auto_linked and self.linked, add brother to self.linked with same parameters
@@ -437,7 +436,7 @@ class TimelineComposition(TimelineSource):
             self.gnlobject.info("insert_source after %s" % existingsource.gnlobject)
         else:
             self.gnlobject.info("insert_source at the beginning")
-            
+
         # find the time where it's going to be added
         if not existingsource or not self._haveGotThisSource(existingsource):
             start = 0
@@ -457,7 +456,7 @@ class TimelineComposition(TimelineSource):
         # set the correct start/duration time
         duration = source.factory.length
         source.setStartDurationTime(start, duration)
-        
+
         # pushing following
         if push_following and not position in [-1, 0]:
             #print self.gnlobject, "pushing following", existorder, len(self.sources[position - 1][2])
@@ -467,16 +466,18 @@ class TimelineComposition(TimelineSource):
                 #print "run", i, "start", mvsrc.start, "duration", mvsrc.duration
                 # increment self.sources[position - 1][i] by source.factory.length
                 mvsrc.setStartDurationTime(mvsrc.start + source.factory.length)
-        
+
         self.addSource(source, position, auto_linked=auto_linked)
 
     def appendSource(self, source, position=1, auto_linked=True):
         """
-        puts a source after all the others
+        puts a source after all the others.
+        position is the layer number
         """
         self.gnlobject.info("source:%s" % source.gnlobject)
+
         # find the source with the highest duration time on the first layer
-        if self.sources[position - 1]:
+        if self.sources[position - 1] and len(self.sources[position - 1][2]):
             existingsource = self.sources[position - 1][2][-1]
         else:
             existingsource = None
@@ -495,7 +496,7 @@ class TimelineComposition(TimelineSource):
         """
         Moves the source to the new position. The position is the existing source before which to move
         the source.
-        
+
         If move_linked is True and the source has a linked source, the linked source will
         be moved to the same position.
         If collapse_neighbours is True, all sources located AFTER the OLD position of the
@@ -510,13 +511,19 @@ class TimelineComposition(TimelineSource):
                                                                                                                    collapse_neighbours))
         sources = self.sources[0][2]
         oldpos = sources.index(source)
-        if newpos == -1:
+
+        # limit newpos to sane values
+        if (newpos == -1) or (newpos > len(sources)):
+            # we want to move the source at the end
             newpos = len(sources)
+        elif (newpos < -1):
+            # we want to move the source at the beginning
+            newpos = 0
 
         self.gnlobject.info("source was at position %d in his layer" % oldpos)
 
         # if we're not moving, return
-        if (oldpos == newpos):
+        if (oldpos == newpos) or ((newpos == len(sources)) and (oldpos == newpos - 1)):
             self.gnlobject.warning("source is already at the correct position, not moving")
             return
 
@@ -557,9 +564,13 @@ class TimelineComposition(TimelineSource):
         self.gnlobject.log("Setting source start position to %s" % gst.TIME_ARGS(newtimepos))
         source.setStartDurationTime(start = newtimepos)
 
-        self.gnlobject.log("Removing source from position [%d] and putting it to position [%d]" % (oldpos, newpos - 1))
+        # take into account the offset of removing the source from the list
+        if oldpos < newpos:
+            newpos -= 1
+
+        self.gnlobject.log("Inserting source to position [%d]" % (newpos))
         del sources[oldpos]
-        sources.insert(newpos - 1, source)
+        sources.insert(newpos, source)
         source.gnlobject.set_property("priority", self.sources[0][0])
 
         # 4. same thing for brother
@@ -572,12 +583,16 @@ class TimelineComposition(TimelineSource):
     def removeSource(self, source, remove_linked=True, collapse_neighbours=False):
         """
         Removes a source.
-        
+
         If remove_linked is True and the source has a linked source, will remove
         it from the linked composition.
         If collapse_neighbours is True, then all object after the removed source
         will be shifted in the past by the duration of the removed source.
+
+        You cannot have remove_linked=False and collapse_neighbours=True !
         """
+        if collapse_neighbours and not remove_linked:
+            raise Exception("You cannot use remove_linked=False and collapse_neighbourse=True")
         self.gnlobject.info("source:%s, remove_linked:%s, collapse_neighbours:%s" % (source, remove_linked, collapse_neighbours))
         sources = self.sources[0]
 
@@ -589,7 +604,8 @@ class TimelineComposition(TimelineSource):
         self.gnlobject.remove(source.gnlobject)
         del sources[2][pos]
 
-        # collapse neighbours
+        # collapse neighbours, changing the start/duration will automatically
+        # change it for the linked sources
         if collapse_neighbours:
             self.gnlobject.info("Collapsing neighbours")
             for i in range(pos, len(sources[2])):
@@ -597,10 +613,14 @@ class TimelineComposition(TimelineSource):
                 obj.setStartDurationTime(start = (obj.start - source.duration))
 
         # if we have a brother
-        if remove_linked and self.linked and self.linked.gnlobject:
-            self.linked.gnlobject.remove(source.linked.gnlobject)
-            self.linked.emit('source-removed', source.linked)
-            self.linked._updateCondensedList()
+        if remove_linked and self.linked and source.linked:
+            sources = self.linked.sources[0]
+            if source.linked in sources[2]: 
+                pos = sources[2].index(source.linked)
+                del sources[2][pos]
+                self.linked.gnlobject.remove(source.linked.gnlobject)
+                self.linked.emit('source-removed', source.linked)
+                self.linked._updateCondensedList()
 
         self.emit('source-removed', source)
         # update the condensed list
