@@ -24,11 +24,6 @@
 
 import gst
 
-def float_framerate(framerate):
-    if framerate.denom == 1:
-        "%d" % framerate.num
-    return "%.2f" % (float(framerate.num) / float(framerate.denom))
-
 def bin_contains(bin, element):
     """ Returns True if the bin contains the given element, the search is recursive """
     if not isinstance(bin, gst.Bin):
